# -*- coding: utf-8 -*-

# Authors: Diego Sarmentero <diego.sarmentero@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Utils Functions Windows specific."""


import ctypes
import os
import subprocess
import sys

# Avoid pylint error on Linux
# pylint: disable=F0401
import win32api
import _winreg
# pylint: enable=F0401

AUTORUN_KEY = r"Software\Microsoft\Windows\CurrentVersion\Run"


def uninstall_application():
    """Uninstall Ubuntu One."""
    if hasattr(sys, "frozen"):
        exec_path = os.path.abspath(sys.executable)
    else:
        exec_path = os.path.dirname(__file__)
    folder = os.path.dirname(exec_path)
    uninstall_path = os.path.join(os.path.dirname(folder), "uninstall.exe")
    if os.path.exists(uninstall_path):
        win32api.ShellExecute(None, 'runas',
            uninstall_path,
            '--mode qt', '', 0)


def start_control_panel():
    """Start the control panel."""
    # If we are in windows and "frozen", assume it's in
    # the same folder as this .exe
    if hasattr(sys, "frozen"):
        cp_path = os.path.join(os.path.dirname(
            os.path.abspath(sys.executable)),
            "ubuntuone-control-panel-qt.exe")
    else:
        cp_path = "ubuntuone-control-panel-qt"
    subprocess.Popen([cp_path, ])


def add_syncdaemon_to_autostart():
    """Add syncdaemon to the session's autostart."""
    if hasattr(sys, "frozen"):
        sd_path = os.path.join(os.path.dirname(
            os.path.abspath(sys.executable)),
            "ubuntuone-syncdaemon.exe")
        u1cp_path = os.path.join(os.path.dirname(
            os.path.abspath(sys.executable)),
            "ubuntuone-control-panel-qt.exe")
    else:
        return
    registry = _winreg.ConnectRegistry(None, _winreg.HKEY_CURRENT_USER)
    key = _winreg.OpenKey(registry,
        AUTORUN_KEY,
        0, _winreg.KEY_ALL_ACCESS)
    # pylint: disable=E0602
    _winreg.SetValueEx(key, "Ubuntu One", 0, _winreg.REG_SZ, sd_path)
    _winreg.SetValueEx(key, "Ubuntu One Icon", 0, _winreg.REG_SZ,
        u1cp_path + " --minimized")
    _winreg.CloseKey(key)


def default_folders():
    """Return a list of the folders to add by default."""
    # XXXX to be replaced by calls to xdg_base_directory's
    # special_folders
    dll = ctypes.windll.shell32
    buf = ctypes.create_string_buffer(300)
    dll.SHGetSpecialFolderPathA(None, buf, 5, False)
    docs = buf.value
    dll.SHGetSpecialFolderPathA(None, buf, 13, False)
    music = buf.value
    dll.SHGetSpecialFolderPathA(None, buf, 39, False)
    pictures = buf.value
    return [docs, music, pictures]
